!+
! Module: omthlcplot_mod
!
! Authors:  Natalie Webb - CESR
!          
! Description: Produce the OM tracking history plot
! 
! Comments: 
! 
! $Id: module.f90,v 1.6 1998/05/27 13:45:39 gvacanti Exp $
!- 
module omthlcplot_mod

  use dal
  use param  
  use errorHandling

  implicit none 
  
  private 

  public omthlcplot

contains 
  
  !---------------------------------------------------------------------
  subroutine omthlcplot
  !---------------------------------------------------------------------
  ! This program reads in the RATES file from the optical monitor and displays the
  ! counts/sec of each column in the file, against time.  The data can also be displayed
  ! binned up and the number of columns to be displayed, can be chosen.
  
    character(len=128) :: Input_File, Output_File, Device_Type
    integer :: Binning_Factor, ncolsintab, nrowsintab, nnewrowsintab, Plots_per_Page
    real(kind=single),dimension(:,:),pointer :: Net_Rate, Err
    real(kind=single) :: Bin_Width, Time_Zero, Time_End
    character(len=1000),dimension(4) :: Label
    real(kind=single),dimension(:),pointer :: Time_Vect

    Input_File = stringParameter("set")
    Device_Type = change_case(stringParameter("plotdevice"),2)
    Output_File = stringParameter("plotfile")
    Binning_Factor = IntParameter("binsize")
    Plots_per_Page = IntParameter("nplots")

    call retrieve_data(Input_File, Bin_Width, Time_Zero, Time_End, Net_Rate, nrowsintab, & 
                                                                        ncolsintab, Label)

    call rebin(Bin_Width, ncolsintab, nrowsintab, Binning_Factor, Net_Rate, nnewrowsintab, & 
                                                                            Time_Vect, Err)  

    call open_plot(Device_Type, Output_File)

    call draw_plot(Label, Time_Vect, Plots_per_Page, ncolsintab, Net_Rate, Err, &
                                                                      nnewrowsintab)

    deallocate(Net_Rate, Time_Vect, Err)

    call pgend

  end subroutine omthlcplot


  !---------------------------------------------------------------------
  subroutine retrieve_data(Input_File, Bin_Width, Time_Zero, Time_End, Net_Rate, &
                                                       nrowsintab, ncolsintab, Label)
  !---------------------------------------------------------------------
  ! The data is read in here and  checks are made to determine whether the correct  
  ! information is contained in the headers

    character(len=128), intent(in) :: Input_File
    type( DataSetT ) :: Ts_Set
    type( TableT ) :: Ts_Table
    real(kind=single),dimension(:,:),pointer :: Net_Rate
    real(kind=single) :: Bin_Width, Time_Zero, Time_End
    integer :: ncolsintab, nrowsintab
    character(len=1000),dimension(4),intent(out) :: Label
 
    ! Read in the fits file
   
    Ts_Set = dataSet(Input_File,READ)
    Ts_Table = table(Ts_Set,BlockNumber(Ts_Set,'RATES'))

    ! Check some of the keywords in the headers

    if ( .not. HasAttribute(Ts_Table,"HDUCLASS") ) &
      call warning('LCPLOT','Keyword HDUCLASS missing in the input file '//Input_File)
    if ( change_case(StringAttribute(Ts_Table,"HDUCLASS"),2) /= 'OGIP' ) &
      call warning('LCPLOT','HDUCLAS is not equal to OGIP')
    if ( .not. HasAttribute(Ts_Table,"HDUCLAS1") ) &
      call warning('LCPLOT','Keyword HDUCLAS1 missing in the input file '//Input_File)
    if ( change_case(StringAttribute(Ts_Table,"HDUCLAS1"),2) /= 'LIGHTCURVE' ) &
      call warning('LCPLOT','HDUCLAS1 is not equal to LIGHTCURVE')
    if ( .not. HasAttribute(Ts_Table,"HDUCLAS2") ) &
      call warning('LCPLOT','Keyword HDUCLAS2 missing in the input file '//Input_File)
    if ( change_case(StringAttribute(Ts_Table,"HDUCLAS2"),2) /= 'TOTAL' ) &
      call warning('LCPLOT','HDUCLAS2 is not equal to TOTAL')
    if ( .not. HasAttribute(Ts_Table,"HDUCLAS3")) &
      call warning('LCPLOT','Keyword HDUCLAS3 missing in the input file '//Input_File)
    if ( change_case(StringAttribute(Ts_Table,"HDUCLAS3"),2) /= 'RATE' ) &
      call warning('LCPLOT','HDUCLAS3 is not equal to RATE')

    if ( .not. HasAttribute(Ts_Table,"TSTART") ) then
      call error('OMTHLCPLOT','Keyword TSTART missing from the input file '//Input_File)
    else
       Time_Zero = real32Attribute(Ts_Table,'TSTART')
    endif

    if ( .not. HasAttribute(Ts_Table,"TIMEDEL") ) then
      call error('OMTHLCPLOT','Keyword TIMEDEL missing from the input file '//Input_File)
    else
      Bin_Width = real32Attribute(Ts_Table,'TIMEDEL')
    endif

    if ( Bin_Width < epsilon(0.) ) then 
      call error('OMTHLCPLOT', 'Input file has null or negative Bin_Width '//Input_File)
    endif

    if ( .not. HasAttribute(Ts_Table,"TSTOP") ) then
      call error('OMTHLCPLOT','Keyword TSTOP missing from the input file '//Input_File)
    else
         Time_End = real32Attribute(Ts_Table,"TSTOP") 
    endif
  
    nrowsintab = numberOfRows(Ts_Table)
    ncolsintab = numberOfColumns(Ts_Table)

    call create_labels (Ts_Set, Label)
    call fill_cols(Ts_Table, nrowsintab, ncolsintab, Net_Rate)
    call release(Ts_Table)
    call release( Ts_Set )

  end subroutine retrieve_data

  !---------------------------------------------------------------------
  subroutine fill_cols(Ts_Table, nrowsintab, ncolsintab, Net_Rate)
  !---------------------------------------------------------------------
  ! The data is read into an array, for easy manipulation throughout the task

    integer, intent(in) :: ncolsintab, nrowsintab
    type( TableT ), intent(in)  :: Ts_Table
    integer :: alltab, i
    character(len=6) :: colname
    character(len=2) :: colno
    real(kind=single),dimension(:),pointer :: Array_Rate 
    real(kind=single),dimension(:,:),pointer :: Net_Rate
    type( ColumnT ) :: Ts_Columns  

    ! Read each column into an array

    allocate(Net_Rate(nrowsintab,ncolsintab))

    do alltab=1,ncolsintab
      if (alltab < 10) then
        write(colno,'(I1)') alltab
      else
        write(colno,'(I2)') alltab
      end if
      colname='RATE'//colno
      Ts_Columns = column(Ts_Table, colname, READ)
      Array_Rate => real32Data(Ts_Columns) 
      therows: do i=1,nrowsintab
        Net_Rate(i,alltab)=Array_Rate(i-1)
      end do therows
    end do
       
  end subroutine fill_cols

  !-------------------------------------------------------------------------------------
  subroutine create_labels(Ts_Set, Label)
  !-------------------------------------------------------------------------------------
  ! Generates labels to add on plot.
 
    type( dataSetT ),intent(in) :: Ts_set
    character(len=1000),dimension(4),intent(out) :: Label
    character(len=12) :: TELESCOP,INSTRUME,DETNAM,DATAMODE,OBSERVER,OBJECT,OBS_ID
    character(len=12) :: DATE_OBS,DATE_END,EXPOSURE,TSTART,TIMEDEL

    type( TableT ) :: Table1 

    Table1 = table(Ts_Set,BlockNumber(Ts_Set,'RATES')) 

    if (HasAttribute(Ts_Set,"TIMEDEL") ) then
         TIMEDEL = StringAttribute(Ts_Set,'TIMEDEL')
    else if (HasAttribute(Table1,"TIMEDEL") ) then
         TIMEDEL = StringAttribute(Table1,'TIMEDEL')
    else
         TIMEDEL = 'Indef'
    endif
 
    if (HasAttribute(Ts_Set,"TSTART") ) then
         TSTART = StringAttribute(Ts_Set,'TSTART')
    else if (HasAttribute(Table1,"TSTART") ) then
         TSTART = StringAttribute(Table1,'TSTART')
    else
         TSTART = 'Indef'
    endif

    if (HasAttribute(Ts_Set,"EXPOSURE") ) then
         EXPOSURE = StringAttribute(Ts_Set,'EXPOSURE')
    else if (HasAttribute(Table1,"EXPOSURE") ) then
         EXPOSURE = StringAttribute(Table1,'EXPOSURE')
    else
         EXPOSURE = 'Indef'
    endif

    if (HasAttribute(Ts_Set,"DATE-END") ) then
         DATE_END = StringAttribute(Ts_Set,'DATE-END')
    else if (HasAttribute(Table1,"DATE-END") ) then
         DATE_END = StringAttribute(Table1,'DATE-END')
    else
         DATE_END = 'Indef'
    endif

    if (HasAttribute(Ts_Set,"DATE-OBS") ) then
         DATE_OBS = StringAttribute(Ts_Set,'DATE-OBS')
    else if (HasAttribute(Table1,"DATE-OBS") ) then
         DATE_OBS = StringAttribute(Table1,'DATE-OBS')
    else
         DATE_OBS = 'Indef'
    endif

    if (HasAttribute(Ts_Set,"OBS_ID") ) then
         OBS_ID = StringAttribute(Ts_Set,'OBS_ID')
    else if (HasAttribute(Table1,"OBS_ID") ) then
         OBS_ID = StringAttribute(Table1,'OBS_ID')
    else
         OBS_ID = 'Indef'
    endif

    if (HasAttribute(Ts_Set,"OBJECT") ) then
         OBJECT = StringAttribute(Ts_Set,'OBJECT')
    else if (HasAttribute(Table1,"OBJECT") ) then
         OBJECT = StringAttribute(Table1,'OBJECT')
    else
         OBJECT = 'Indef'
    endif

    if (HasAttribute(Ts_Set,"OBSERVER") ) then
         OBSERVER = StringAttribute(Ts_Set,'OBSERVER')
    else if (HasAttribute(Table1,"OBSERVER") ) then
         OBSERVER = StringAttribute(Table1,'OBSERVER')
    else
         OBSERVER = 'Indef'
    endif

   if (HasAttribute(Ts_Set,"DATAMODE") ) then
         DATAMODE = StringAttribute(Ts_Set,'DATAMODE')
    else if (HasAttribute(Table1,"DATAMODE") ) then
         DATAMODE = StringAttribute(Table1,'DATAMODE')
    else
         DATAMODE = 'Indef'
    endif

    if (HasAttribute(Ts_Set,"DETNAM") ) then
         DETNAM = StringAttribute(Ts_Set,'DETNAM')
    else if (HasAttribute(Table1,"DETNAM") ) then
         DETNAM = StringAttribute(Table1,'DETNAM')
    else
         DETNAM = 'Indef'
    endif

    if (HasAttribute(Ts_Set,"INSTRUME") ) then
         INSTRUME = StringAttribute(Ts_Set,'INSTRUME')
    else if (HasAttribute(Table1,"INSTRUME") ) then
         INSTRUME = StringAttribute(Table1,'INSTRUME')
    else
         INSTRUME = 'Indef'
    endif

    if (HasAttribute(Ts_Set,"TELESCOP") ) then
         TELESCOP = StringAttribute(Ts_Set,'TELESCOP')
    else if (HasAttribute(Table1,"TELESCOP") ) then
         TELESCOP = StringAttribute(Table1,'TELESCOP')
    else
         TELESCOP = 'Indef'
    endif

    Label(1) = trim(TELESCOP)//' - '// &
               trim(INSTRUME)//' - '// &
               trim(DETNAM)//' - '// &
               trim(DATAMODE)// &
               ' - PI NAME : '//trim(OBSERVER)// &
               ' - OBJECT : '//trim(OBJECT)// &
               ' - OBS_ID : '//trim(OBS_ID)
    Label(2) = 'START : '//trim(DATE_OBS)//' - '// &
               'STOP : '//trim(DATE_END)
    Label(3) ='EXPOSURE DURATION='//trim(EXPOSURE)//' secs '&
         //' - TIME ZERO : '//trim(TSTART)//' MJD' &
         //' - ORIGINAL BIN SIZE : '//trim(TIMEDEL)//' secs'

    Label(4) = '---------------'

  end subroutine create_labels

  !-------------------------------------------------------------------------------------
  subroutine rebin(Bin_Width, ncolsintab, nrowsintab, Binning_Factor, Net_Rate, & 
                                                    nnewrowsintab, Time_Vect, Err)
  !-------------------------------------------------------------------------------------
  ! Rebins Net_Rate to ensure good signal-to-noise and to limit the number of 
  ! plotted points to 500.  Also modifies Time_Vect.

    real(kind=single),intent(in) :: Bin_Width
    real(kind=single),dimension(:,:),pointer :: Net_Rate, Net_Rate_Reb, Err
    integer,intent(inout) :: nrowsintab, ncolsintab, Binning_Factor
    integer,intent(out) :: nnewrowsintab
    integer :: j, k, thetable, i
    real(kind=single),dimension(:),pointer :: Time_Vect

    ! Sets Binning_Factor to correct value

    if (Binning_Factor < 1) then
       Binning_Factor = 1
       call warning('OMTHLCPLOT','Binning_Factor must be positive, set to default value : 1')
    end if
    if ( int(nrowsintab/Binning_Factor)>500 ) then
       Binning_Factor = (nrowsintab-1)/500 + 1
       write(errstr,*) 'Data points to plot must not exceed 500. '//&
        'The binning factor has been increased from the input value to : ',Binning_Factor
       call warning('OMTHLCPLOT')
    end if

    nnewrowsintab=int(nrowsintab/Binning_Factor)

    ! Rebinning rate, errors (rate**0.5) and the time 

    allocate(Net_Rate_Reb(nnewrowsintab,ncolsintab), Err(nnewrowsintab,ncolsintab))  
    allocate(Time_Vect(nnewrowsintab))    

    Net_Rate_Reb=0.0; Err=0.0

    binthem: do thetable=1,ncolsintab
      i=1; j=1; Time_Vect=0.0
      do 
        if (i > nrowsintab-Binning_Factor+1) exit
          do k=i,i+Binning_Factor-1
              Time_Vect(j)=Time_Vect(j)+(Bin_Width*0.5)
            if (Net_Rate(k,thetable) > 0.0) then
              Err(j,thetable)=Err(j,thetable)+(1.0/(Net_Rate(k,thetable)))
              Err(j,thetable)=1.0/sqrt(Err(j,thetable))
            else 
              Err(j,thetable)=0.0
            endif
            Net_Rate_Reb(j,thetable)=Net_Rate_Reb(j,thetable)+Net_Rate(k,thetable)
          end do
         i=i+Binning_Factor
         j=j+1
      end do
    end do binthem

    ! Makes the Time_Vect cumulative for plotting

    do i=1,nnewrowsintab
      if (i>1) then
        Time_Vect(i)=Time_Vect(i)+Time_Vect(i-1)
      else
        Time_Vect(i)=Time_Vect(i)
      end if
    end do

    Net_Rate_Reb=Net_Rate_Reb/real(Binning_Factor)    

    deallocate(Net_Rate)
    allocate(Net_Rate(nnewrowsintab,ncolsintab))
    Net_Rate=Net_Rate_Reb
    deallocate(Net_Rate_Reb)  
 
  end subroutine rebin

  !-------------------------------------------------------------------------------------
  subroutine open_plot(Device_Type, Output_File) 
  !-------------------------------------------------------------------------------------
  ! Check Device_Type and Output_File. Open plot if required.

    character(len=*),intent(inout) :: Device_Type , Output_File
    integer :: pgbeg
    character(len=2048) :: Device_Selected
    logical :: File_Exists

    if (Output_File == '')  Output_File='default.ps'
    if (Device_Type(1:3) /= '/XW') then
      inquire(file=Output_File,exist=File_Exists)   
      if (File_Exists .and. .not. clobber()) call error('OMTHLCPLOT', & 
         'File already exists' & 
         //trim(Output_File)//' and can not be overwritten (SAS_CLOBBER=0)')
      if (File_Exists .and. clobber()) call warning('OMTHLCPLOT', &
         'File already exists '//trim(Output_File)//' but will be overwritten')
        Device_Selected=trim(Output_File)//trim(Device_Type)
      if ( pgbeg(0,Device_Selected,1,1)/=1 ) then
        call warning('OMTHLCPLOT', 'Invalid device and/or output file : ' & 
          //trim(Device_Type)//' , '//trim(Output_File))
        Output_File='default.ps'; Device_Type= '/VPS'    
        Device_Selected=trim(Output_File)//trim(Device_Type)
        if ( pgbeg(0,Device_Selected,1,1)/=1 ) then
          call error('OMTHLCPLOT', 'Invalid DEFAULT device and/or output file : ' &
            //trim(Device_Type)//' , '//trim(Output_File))
        else
          call warning('OMTHLCPLOT', &
          'New device and output file : '//trim(Device_Type)//' , '//trim(Output_File))
        endif
      endif
      else
        Device_Selected=trim(Device_Type)
      if ( pgbeg(0,Device_Selected,1,1)/=1 ) call error('OMTHLCPLOT', &
        'Invalid device : '//trim(Device_Type))
    endif
    call pgpap(7.0, 1.4)
    call pgask(.true.)
    call pgsch(0.5)

  end subroutine open_plot

  !-------------------------------------------------------------------------------------
  subroutine draw_plot(Label, Time_Vect, Plots_per_Page, ncolsintab, Net_Rate, Err, &
                                                                          nnewrowsintab)
  !-------------------------------------------------------------------------------------
  ! Draw labelled plot. A Nlight pages one, in which each page contains the net and
  ! background source time serie representation of a specific light-curve.

    character(len=1000),dimension(4),intent(in) :: Label
    real(kind=single),dimension(:),pointer :: Time_Vect
    real(kind=single),dimension(:,:),pointer :: Net_Rate, Err
    real, dimension(100) :: group
    integer,intent(in) :: nnewrowsintab, ncolsintab, Plots_per_Page

  ! Local variables Window1&2 are used to define the viewports.

    real(kind=single),dimension(4,ncolsintab) :: Windows
    integer :: Light, i, j, k

    Windows=0.0; k=1

    do i=1,100
      group(i)=real(i)
    end do

    ! Define the size and location of the plots for each page of the output file

    do i=1,ncolsintab
      do j=1,100
        if (real(i-1)/real(Plots_per_Page) == group(j)) then
           k=1 
        endif
      end do
      Windows(1,i) = 0.15
      Windows(2,i) = 0.95
      Windows(3,i) = k*(1.0/real(Plots_per_Page+1))-(1.0/(2.0*real(Plots_per_Page+1)))
      Windows(4,i) = k*(1.0/real(Plots_per_Page+1))
      k=k+1
    end do

    do Light=1,ncolsintab
       do j=1,100
         if (real(Light-1)/real(Plots_per_Page) == group(j)) then
           call pgpage
         endif
       end do
       call pgsvp(0.0,1.0,0.0,1.0)
       call pgsci(10)
       call pgsch(0.6)       
       do i=1,4
          call pgmtxt('L' , -(1. + (i-1)*1.5) , 0.5 , 0.5 , Label(i))
       end do
       call pgsci(11)
       call pgsch(0.5)
       call make_one_plot(Windows(:,Light), Time_Vect, Net_Rate(:,Light), Err(:,Light), & 
                                                          nnewrowsintab, Light)
    end do

  end subroutine draw_plot

  !-------------------------------------------------------------------------------------
  subroutine make_one_plot(Window, Time_Vect, Rate, Error, nnewrowsintab, Light)
  !-------------------------------------------------------------------------------------
  ! Draw one specific plot. X-axis corresponds to time and Y-axis to count rates.
  ! Plot title and mean rate are labelled.

    real(kind=single),dimension(:),pointer :: Time_Vect
    real(kind=single),dimension(:) :: Rate, Error
    integer,intent(in) :: nnewrowsintab, Light
    real(kind=single),dimension(4),intent(in) :: Window

    character(len=20) :: String
    integer :: Length
    character(len=2) :: colno
  
    call string_of_real(sum(Rate)/real(nnewrowsintab), String, Length )
    call pgsvp(Window(1),Window(2),Window(3),Window(4))
    call pgswin(0., Time_Vect(nnewrowsintab) + Time_Vect(1), &
         min(0.,minval(Rate)-sqrt(maxval(Rate))), maxval(Rate)+sqrt(maxval(Rate))+0.5 )
    call pgsci(1)
    call pgbox('BCNST', 0.0, 0.0, 'BCNST', 0.0, 0.0)
    call pglab('Time after TIMESTART (sec)' , 'Counts/sec', ' ')
    write(colno,'(I2)') Light
    call pgmtxt('T' , 0.5 , 1.0 , 1.0 , 'RATES ' //colno)
    call pgsci(2)
    call pgerry(nnewrowsintab, Time_Vect, Rate+Error, Rate-Error, 0.0)
    call pgsci(1)
    call pgslw(10)
    call pgpt(nnewrowsintab, Time_Vect, Rate, -1)
    call pgslw(1)

  end subroutine make_one_plot

  !-------------------------------------------------------------------------------------
  subroutine string_of_real(Nb , String , Length)
  !-------------------------------------------------------------------------------------
  ! Converts real numbers into plottable character chain. Gives also its length.

    real(kind=single),intent(in) :: Nb
    character(len=20),intent(out) :: String
    integer,intent(out) :: Length
    integer :: MM , puiss

    puiss = max( 0 , min( 6 , 6-int(log10(Nb)) ) )
    MM = nint(Nb*(10**puiss))
    call pgnumb(MM , -puiss , 0 , String , Length)

  end subroutine string_of_real

  !-------------------------------------------------------------------------------------
  function change_case(instring,dir) result(outstring)
  !-------------------------------------------------------------------------------------
  ! Switch from lower to upper case or vice-versa: From Jean Ballet
  ! instring : input string
  ! dir      : direction (1: to lower case; 2: to uppercase; 0: switch case)

    character(len=*), intent(in) :: instring
    integer, intent(in) :: dir
    character(len=len(instring)) :: outstring

  ! ASCII positions of lower case and upper case letters

    integer, parameter :: start_up  = iachar("A"), end_up  = iachar("Z"), &
                          start_low = iachar("a"), end_low = iachar("z"), &
                          offset = start_low - start_up

    integer :: length, icur, pos

    length = len(instring)
    outstring = instring
    do icur = 1, length
       pos = iachar(instring(icur:icur))
       if (pos >= start_up .and. pos <= end_up .and. dir < 2) then
          pos = pos + offset
       else if (pos >= start_low .and. pos <= end_low .and. dir /= 1) then
          pos = pos - offset
       endif
       outstring(icur:icur) = achar(pos)
    enddo

  end function change_case  
   
end module omthlcplot_mod

